%In this Computational Exploration we will load a series of images of a
%movie of a growing E. coli colony. We will measure its growth by
%calculating the area of the colony. In order to do this we'll resort to
%thresholding just like we did in the Computational Exploration in chapter
%2.

%Let's start by looking at the first frame of the movie:
Colony=imread('EcoliGrowth1.tif');
imshow(Colony,[])

%Notice that the cells can be relatively well separated from the background
%because they are much darker. In order to select a treshold that will
%perform such separation we normalize the image such that its brightest
%pixel is 1 and its dimmest pixel is 0. This is done using the function
%"mat2gray"
ColonyNorm=mat2gray(Colony);
%Notice that the image looks exactly the same
imshow(ColonyNorm)
%but compare the pixel values of each image
[min(min(Colony)),max(max(Colony))]
[min(min(ColonyNorm)),max(max(ColonyNorm))]

%Now, let's figure out a new threshold. We encourage the reader to play
%around with the value to get a feel for what works best.
Threshold=0.4;
ColonyThresh=ColonyNorm <Threshold;
imshow(ColonyThresh)

%In order to calculate the area of the colony we just need to sum over all
%white pixels
Area=sum(sum(ColonyThresh))

%Now that we're happy with the threshold chosen let's do this for all
%images. Each frame will be processed in a for-loop. In each instance we will
%use the command "pause" to give us an oportunity to look at the images.

for i=1:35
    %Load the file. Notice the combination of a plain string
    %("EcoliGrowth") with the command "num2str", which turns the index "i"
    %into a string
    Colony=imread(['EcoliGrowth',num2str(i),'.tif']);
    
    ColonyNorm=mat2gray(Colony);
    ColonyThresh=ColonyNorm<Threshold;
    
    %Plot the raw and the thresholded images side by side
    montage([ColonyNorm,ColonyThresh])
    
    Area(i)=sum(sum(ColonyThresh));
    
    %Uncomment this line if you want to save the thresholded images
    %imwrite(ColonyThresh,['EcoliGrowth',num2str(i),'-Thresh.tif'],'Compression','None')
    
    %Wait for the user to press a key
    pause
end

%You might have noticed that as the colony grows the contrast becomes
%worse. This is a limitation of the simple strategy adopted here.

%The frames of the movie were taken every ten minutes. We then can plot the
%colony area as a function of time 
Time=0:10:340;

figure(2)
plot(Time,Area)
xlabel('time (min)')
ylabel('area (pixels^2)')

%It is even more interesting to plot this on a log-scale for the y-axis
figure(3)
semilogy(Time,Area)
xlabel('time (min)')
ylabel('area (pixels^2)')

%It is surprinsigly straight despite the roughness of the segmentation
%algorithm. Let's find the doubling time by doing a linear fit to the
%natural logarithm of the area. If we expect the area to follow an
%exponential growth then we can describe it as
%Area = A0 * exp(k*t),
%where A0 is the initial area and k is a division rate. If we
%fit the log of the area we expect
%log(Area) = log(A0) + k*t.
%As a result the slope of a linear fit gives us k, which is related to
%the doubling time by tdouble=ln(2)/k.
%Let's do a linear fit
FitResults=polyfit(Time,log(Area),1);
Slope=FitResults(1);
Intercept=FitResults(2);

%To calculate the doubling time we then carry out the operation
DoublingTime=1/Slope*log(2);
%Which is on the order of 52 minutes! These cells were grown in minimal
%media. Under such conditions we expect them to grow slower than in rich
%media such as LB, where they would divide approximately every 30 minutes.
%Now, let's plot this together with
%the data
figure(4)
semilogy(Time,Area,'.k')
hold on
semilogy(Time,exp(Intercept)*exp(Slope*Time),'-k')
hold off
xlabel('time (min)')
ylabel('area (pixels^2)')

%We see that the growth of the colony follows relatively closely an
%exponential trend. Finally, we can plot this all in units of cells. We
%know that in the first frame we had two cells. Let's normalize the are by
%this magnitude
NCells=Area/(Area(1)/2);

figure(5)
semilogy(Time,NCells,'.k')
hold on
semilogy(Time,exp(Intercept)*exp(Slope*Time)/(Area(1)/2),'-k')
hold off
ylabel('number of cells')
xlabel('time (min)')
